package eu.maiora.db

import eu.maiora.model.*
import kotlinx.coroutines.Dispatchers
import org.jetbrains.exposed.dao.IntEntity
import org.jetbrains.exposed.dao.IntEntityClass
import org.jetbrains.exposed.dao.LongEntity
import org.jetbrains.exposed.dao.LongEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IdTable
import org.jetbrains.exposed.sql.Transaction
import org.jetbrains.exposed.sql.javatime.datetime
import org.jetbrains.exposed.sql.transactions.experimental.newSuspendedTransaction
import java.time.format.DateTimeFormatter



object AccountsTable : IdTable<Int>("accounts"){
    override val id = integer("id").entityId()
    val username = varchar("username", 255)
    val password = varchar("password", 255)

    override val primaryKey = PrimaryKey(id)
}

object ParametriTable : IdTable<Int>("parametri"){
    override val id = integer("id").entityId()
    val chiave = varchar("chiave", 255)
    val valore = varchar("valore", 255)

    override val primaryKey = PrimaryKey(id)
}

object TessereTable : IdTable<Long>("view_tessere_api"){
    override val id = long("id").entityId()
    val idUtente = long("id_utente")
    val codiceFiscale = varchar("codice_fiscale", 255)
    val numero = varchar("numero", 255)
    val saldo = double("saldo")
    val punti = integer("punti")

    override val primaryKey = PrimaryKey(id)
}

object MovimentiTable : IdTable<Long>("view_movimenti_api"){
    override val id = long("id").entityId()
    val idTessera = long("id_tessera")
    val numero = varchar("numero_tessera", 255)
    val operazione = varchar("operazione", 255)
    val dataMovimento = datetime("data_movimento")
    val transazione = varchar("transazione", 255)
    val dispositivo = varchar("dispositivo", 255)
    val riferimento= varchar("riferimento", 255)
    val saldoPre = double("saldo_pre")
    val importo = double("importo")
    val saldoPost = double("saldo_post")
    val puntiPre = integer("punti_pre")
    val punti = integer("punti")
    val puntiPost = integer("punti_post")
}

object ComposizioniTable : IdTable<Long>("view_composizioni"){
    override val id = long("id").entityId()
    val idVassoio = long("id_vassoio")
    val vassoio = varchar("vassoio", 255)
    val idCategoria = long("id_categoria")
    val categoria = varchar("categoria", 255)
    val quantita = integer("quantita")
}

object RicaricheTable : IdTable<Long>("ricariche_app"){
    override val id = long("id").autoIncrement("seq_ricariche_app").entityId()
    val codiceTransazione = varchar("codice_transazione", 255)
    val messaggio = varchar("messaggio", 255).nullable()
    val idTessera = long("id_tessera")
    val importo = double("importo")
}

class AccountsDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<AccountsDAO>(AccountsTable)

    var username by AccountsTable.username
    var password by AccountsTable.password
}

class ParametriDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<ParametriDAO>(ParametriTable)

    var chiave by ParametriTable.chiave
    var valore by ParametriTable.valore
}

class TessereDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<TessereDao>(TessereTable)

    var idUtente by TessereTable.idUtente
    var codiceFiscale by TessereTable.codiceFiscale
    var numero by TessereTable.numero
    var saldo by TessereTable.saldo
    var punti by TessereTable.punti
}

class MovimentiDao(id: EntityID<Long>) :LongEntity(id) {
    companion object : LongEntityClass<MovimentiDao>(MovimentiTable)

    var idTessera by MovimentiTable.idTessera
    var numero by MovimentiTable.numero
    var operazione by MovimentiTable.operazione
    var dataMovimento by MovimentiTable.dataMovimento
    var transazione by MovimentiTable.transazione
    var dispositivo by MovimentiTable.dispositivo
    var riferimento by MovimentiTable.riferimento
    var saldoPre by MovimentiTable.saldoPre
    var importo by MovimentiTable.importo
    var saldoPost by MovimentiTable.saldoPost
    var puntiPre by MovimentiTable.puntiPre
    var punti by MovimentiTable.punti
    var puntiPost by MovimentiTable.puntiPost
}

class ComposizioniDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<ComposizioniDao>(ComposizioniTable)

    var idVassoio by ComposizioniTable.idVassoio
    var vassoio by ComposizioniTable.vassoio
    var idCategoria by ComposizioniTable.idCategoria
    var categoria by ComposizioniTable.categoria
    var quantita by ComposizioniTable.quantita
}

class RicaricheDao(id: EntityID<Long>) :LongEntity(id){
    companion object : LongEntityClass<RicaricheDao>(RicaricheTable)

    var codiceTransazione by RicaricheTable.codiceTransazione
    var messaggio by RicaricheTable.messaggio
    var idTessera by RicaricheTable.idTessera
    var importo by RicaricheTable.importo
}


fun accountsDaoToModel(dao: AccountsDAO) = Accounts(
    dao.id.value,
    dao.username,
    dao.password
)

fun parametriDaoToModel(dao: ParametriDAO) = Parametri(
    dao.id.value,
    dao.chiave,
    dao.valore
)

fun tessereDaoToModel(dao: TessereDao) = Tessere(
    dao.id.value,
    dao.idUtente,
    dao.codiceFiscale,
    dao.numero,
    dao.saldo,
    dao.punti
)

fun movimentiDaoToModel(dao: MovimentiDao) :Movimenti{
    val formatter = DateTimeFormatter.ofPattern("ddMMyyyy HH:mm")
    val formattedDate = dao.dataMovimento.format(formatter)

    return Movimenti(
        dao.id.value,
        dao.idTessera,
        dao.numero,
        dao.operazione,
        formattedDate,
        dao.transazione,
        dao.dispositivo,
        dao.riferimento,
        dao.saldoPre,
        dao.importo,
        dao.saldoPost,
        dao.puntiPre,
        dao.punti,
        dao.puntiPost
    )
}

fun composizioniDaoToModel(dao: ComposizioniDao) = Composizioni(
    dao.id.value,
    dao.idVassoio,
    dao.vassoio,
    dao.idCategoria,
    dao.categoria,
    dao.quantita
)


suspend fun <T> suspendTransaction(block: Transaction.() -> T): T =
    newSuspendedTransaction(Dispatchers.IO, statement = block)